/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef WINTVSCREEN_H
#define WINTVSCREEN_H

#include "wintv.h"
#include "xrwevent.h"

// includes needed by qt
#include <qapp.h>
#include <qwidget.h>
#include <qtimer.h>
#include <qdatetm.h> 
#include <qlcdnum.h>
#include <qpopmenu.h>

// video clip stuff
#include "videoclip.h"

// includes needed by bttv
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

// additional
#include "channel.h"

// ---- bttv stuff -----
#ifndef NOCARD
#define __KERNEL__
#ifdef _I386_TYPES_H
#undef _I386_TYPES_H
#endif
#include <linux/types.h>
#include <videodev.h>
#undef __KERNEL__
#include <bttv.h>
#endif

// debug switches
#ifdef myDEBUG
#define tvDEBUG
//#define X11DEBUG
#endif

/* ------------------- some definitions ---------------------------------*/

// input norm
#define PAL 0
#define NTSC 1
#define SECAM 2

// input source
#define TUNER 0
#define COMP1 1
#define COMP2 2
#define SVHS  3

// aspect ratio
#define ASPECT_NONE  0
#define ASPECT_16_9  1
#define ASPECT_4_3   2

// some defaults 
#define DCOL 254
#define DBRI 0
#define DHUE 0
#define DCONTRAST 216
#define DNORM  PAL
#define DIN TUNER
#define DFREQ 772
#define DDEV "/dev/bttv"
#define DCHANNEL 1

#define PICNAME "pic"

#define NONE 0
#define UPPER_LEFT 1
#define UPPER_RIGHT 2
#define LOWER_LEFT 3
#define LOWER_RIGHT 4

#define DEFAULT_WIDTH    320
#define DEFAULT_HEIGHT   240  

#define PAL_WIDTH 768
#define PAL_HEIGHT 576

#define PAL_MAX_WIDTH 800
#define PAL_MAX_HEIGHT 600

#define NTSC_WIDTH 640
#define NTSC_HEIGHT 480

#define NTSC_MAX_WIDTH 640
#define NTSC_MAX_HEIGHT 480

// several delay times 
#define DEFAULT_DELAY 100       // 0.1 sec
#define MOUSE_DELAY 10000       // 10 sec.
#define LCD_DELAY 5000          // 5 sec
#define AUDIOMODE_DELAY 500     // 0.5 sec.

// audio 
#define AUDIOMODE_AUTODETECT 256
#define CAN_AUDIO_VOLUME     1

// supported video clip formats
#define VIDEOCLIP_AVI 1
#define VIDEOCLIP_PPM 2
#define VIDEOCLIP_RAW 3

//============================================================================================
struct STRTAB {
    int  nr;
    char *str;
};

static char *device_cap[] = {
    "capture", "tuner", "teletext", "overlay", "chromakey", "clipping",
    "frameram", "scales", "monochrome", NULL
};

static char *device_pal[] = {
    "-", "grey", "hi240", "rgb565", "rgb24", "rgb32", "rgb555", NULL
};

static struct STRTAB norms[] = {
    {  0, "PAL" },
    {  1, "NTSC" },
    {  2, "SECAM" },
    {  3, "AUTO" },
    { -1, NULL }
};

static struct STRTAB *inputs;

static struct STRTAB audiomode[] = {
    {  0, "auto"    },
    {  1, "mono"    },
    {  2, "stereo"  },
    {  3, "lang1"   },
    {  4, "lang2"   },
    { -1, NULL,     },
};


//========================= class winTVScreenWidget =====================================

class winTVScreenWidget : public QWidget
{
  
  Q_OBJECT

public:
  winTVScreenWidget( QWidget *parent=0, 
		     const char *name=0,
		     int _capture=0, 
		     int _automouseoff=1,
		     char *_bttvdev=DDEV, 
		     int _bpp=0,int _dual=0,
		     int _freq=DFREQ, 
		     int _input=DNORM, 
		     char *_visualname=0
		     );

  ~winTVScreenWidget();
  
  int win_default_width(void) { return DEFAULT_WIDTH; }
  int win_default_height(void) { return  DEFAULT_HEIGHT; }
  int win_max_width(void) { return PAL_WIDTH; }
  int win_max_height(void) { return PAL_HEIGHT; }

  int getScreenSaverState() { return saverState; }
  int getCaptureState() { return captureState; }
  int getCursorState() { return cursorState; }

  int getMuteState() { 
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCGCAUDIO"); 
    return (vaudio[0].flags & VIDEO_AUDIO_MUTABLE) ? (vaudio[0].flags&VIDEO_AUDIO_MUTE) : -1;
  }
  int getAudioMode() { 
    return audioMode;
  }
  int getVolumeState() {
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCGCAUDIO");  
    return (vaudio[0].flags & VIDEO_AUDIO_VOLUME) ? vaudio[0].volume : -1;
  }

  struct video_capability* getCapCap() { return &vcap; }
  struct video_channel* getCapChannel() { return vchan; }
  struct video_audio* getCapAudio() { return vaudio; }
  struct video_tuner* getCapTuner() { return &vtuner; }
  
  void setTVPopup(QPopupMenu *tvp) { tvPopup=tvp; } 
  void destroyTV();

  /* -------------------------------------------------------------------------------------- */
  /* -------------------------------------------------------------------------------------- */
public slots:
  // use this slots to change anything 

  // capture
  void changeCaptureSLOT(int value);       // toggle capture on/off; init is set by constructor

  // tunig et al
  void changeFrequencySLOT(int value);     // does what its name says ...
  void changeBrightnessSLOT(int bri);      
  void changeContrastSLOT(int contr);
  void changeColorSLOT(int color);
  void changeHueSLOT(int hue);
  void changeSourceSLOT(int s);
  void changeNormSLOT(int n);

  // set all 
  void changeChannelValuesSLOT(channel *c);

  // aspect ratio
  //void changeAspectRatioSLOT(int a);     // obsolete
  void changeDelaySLOT(int d);             // unused

  // sound et al
  void toggleAudioMuteSLOT();                 // toggles mute on/off, init is on
  void changeAudioVolumeSLOT(int vl, int vr);     // 
  void changeAudioModeSLOT(int mode);         // init is auto

  // screen saver and mouse cursor
  void toggleScreenSaverSLOT();      // toggles screen saver on/off; init is on
  void toggleMouseCursorSLOT();      // toggles mouse cursor on/off; init is on

  // unused stuff
  void changeTVMultiDisplaySLOT(int x, int y, int w, int h); // not used yet

  // one snap shot
  void snapShotSLOT(int FULL=1, int count=1);

  // create video clip; format: VIDEOCLIP_AVI, VIDEOCLIP_PPM, VIDEOCLIP_RAW
  void startVideoClipSLOT(struct MOVIE_PARAMS *par); 
  void endVideoClipSLOT();

  /* -------------------------------------------------------------------------------------- */
  /* -----stop reading --- everything below is private ... -------------------------------- */
  /* -------------------------------------------------------------------------------------- */
private slots:
  void refreshTVActionSLOT();
  void autoDetectAudioModeSLOT();
  void writeVideoClipFrameSLOT();

  /* -------------------------------------------------------------------------------------- */
signals:
  void keyPressedSIGNAL(int key);   // returns all keys pressed in winTVScreenWidget
  void autoDetectAudioModeSIGNAL(int mode);
  void wroteVideoClipFrameSIGNAL(int frames);
  void lostVideoClipFrameSIGNAL(int frames);
  void statVideoClipFrameSIGNAL(double);

  /* -------------------------------------------------------------------------------------- */
protected:
  void resizeEvent( QResizeEvent * );
  void moveEvent( QMoveEvent * );
  //void paintEvent (QPaintEvent * );
  //void closeEvent (QCloseEvent * );
  bool eventFilter( QObject *, QEvent * );
  bool x11Event( XEvent * );
  void keyPressEvent ( QKeyEvent *);
  //void mouseMoveEvent (QMouseEvent *);
  void mouseReleaseEvent ( QMouseEvent * me);

  /* -------------------------------------------------------------------------------------- */
private:
  void capture(int on); // 1=on 0=off
  void switch_capture(int on); // switch capture on/off 

  int initializeTV();

  void newTVGeometry(int x=0, int y=0, int w=0, int h=0);
  void setTVMaxSize();

  void refreshTV();

  void addTVClip(struct video_clip *cr,int *ncr,uint x,uint y,uint x2,uint y2);
  void makeTVClipTab(int ncr);
  int getTVClip(void);

  //bool onScreen(XWindowAttributes wts);
  //void bt848_dma(unsigned int);                        

/* -------------------------------------------------------------------------------------- */
/* ---------------------------- resources ----------------------------------------------- */
/* -------------------------------------------------------------------------------------- */

  char *devname;           // name of the bttv-device
  int fbttv;               // the bttv-device

  // bttv stuff
  bool CAP_ON;          // picture on/off
  int NORM;                // input norm, PAL, NTSC, SECAM
  int FREQ;                // frequency, *16 
  int INPUT;               // input channel, TUNER, COMP1, COMP2, SVHS 
  int DELAY;               // 200ms
  int Bright;              // Brightness
  int Color;               // intensity of colors
  int Hue;                 // hue
  int Contrast;            // contrast
  
  bool snap;            // snapshot

  // video 4 linux 
#ifndef NOCARD
  struct video_window vwin;  // bttv structure 
  struct video_buffer vbuf;  // bttv structure 
  struct video_tuner vtuner; // bttv structure 
  struct video_capability vcap; 
  struct video_channel *vchan;
  struct video_picture vpic;
  struct video_audio *vaudio;
  struct video_clip cliprecs[MAX_CLIPRECS];    // list of clips
  struct video_mmap vgrab_even,vgrab_odd;
  char *TargetMem;         // memory for snapshot  (was: unchar)
  int msize;               // size of memory for snapshot 
#endif

  uint max_width;          // max width of TV picture 
  uint max_height;         // max height of TV picture, both set by setTVMaxSize

  bool Change;          // internal, size of TV picture changed
  bool Fresh;           // TV picture needs refresh done by refreshTV()

  // color and visual stuff
  //Visual *visual;          // the aktual visual
  char *visname;           // name of visual
  // FIX ME
  //XColor colors[256];
  //Colormap colormap;       // colormap, needed when having only 256 colors -> pseudocolor
  int alloc_context;
  QColor qcolors[256];     // colormap, needed when having only 256 colors -> pseudocolor
  // color depth etc
  bool dual;            // have grafic-card with dual-buffer ( 32 bpp mode ?? )
  int numbitplanes;        // number of bitplanes 
  int bpp;                 // bytes per pixel

  int realw;               // real width of screen (not used yet)
  int aspect;              // aspect ratio (4:3 or 16:9) (not used yet)

  bool CMSET;           // are we using an own colormap ( <= 256 colors)
  bool REFRESH;         // do we want refreshs (always TRUE till now) 

  bool wasCaptureOn;    // whats its name says

  bool Move;            // the widget was moved

  int is_aktive;           // the widget is aktive

  QCursor orgCursor;       // Cursor

  //QTimer *mtimer;          // timer for deactivating mouse pointer
  int autoMouseOFF;        // deactivating mouse pointer

  QTimer *audioDetectTimer;
  int audioMode,audioVolumeCan,audioVolume;
  
  int cursorState,saverState,muteState,captureState,showchannelState;

  int timeout,interval,prefer_blanking,allow_exposures; // stuff for ScreenSaver

  int shotnumber;          // number of screen shot, starting at 0

  bool timer_inactive;

  // popup
  QPopupMenu *tvPopup;

  // video clip stuff
  QTime videoTime;
  videoClip *video;
  int video_aktive, video_capturing,video_frame_even;
  long elapsed;
};

#endif WINTVSCREEN_H
















